--[[
    Armory Addon for World of Warcraft(tm).
    Revision: 113 2009-10-10T12:18:36Z
    URL: http://www.wow-neighbours.com

    License:
        This program is free software; you can redistribute it and/or
        modify it under the terms of the GNU General Public License
        as published by the Free Software Foundation; either version 2
        of the License, or (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program(see GPL.txt); if not, write to the Free Software
        Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    Note:
        This AddOn's source code is specifically designed to work with
        World of Warcraft's interpreted AddOn system.
        You have an implicit licence to use this AddOn with these facilities
        since that is it's designated purpose as per:
        http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]]

local Armory = Armory;
local container = "Quests";

local selectedQuestLine = 0;
local questLogFilter = "";

----------------------------------------------------------
-- Quests Internals
----------------------------------------------------------

local questLines = {};
local dirty = true;
local owner = "";

local function GetQuestLines()
    local dbEntry = Armory.selectedDbBaseEntry;
    local count = dbEntry:GetNumValues(container);
    local collapsed = false;
    local hasItems = (questLogFilter == "");
    local include, text, numItems;

    table.wipe(questLines);
    
    for i = 1, count do
        local name, _, _, _, isHeader = dbEntry:GetValue(container, i, "Info");
        local isCollapsed = Armory:GetHeaderLineState(container, name);
        if ( isHeader ) then
            table.insert(questLines, i);
            collapsed = isCollapsed;
        elseif ( not collapsed ) then
            if ( questLogFilter == "" ) then
                include = true;
            else
                text = name.."\t"..dbEntry:GetValue(container, i, "Text");
                numItems = dbEntry:GetValue(container, i, "NumLeaderBoards") or 0;
                if ( numItems > 0 and dbEntry:Contains(container, i, "LeaderBoards") ) then
                    for index = 1, numItems do
                        text = text.."\t"..(dbEntry:GetValue(container, i, "LeaderBoards", index) or "");
                    end
                end
                numItems = dbEntry:GetValue(container, i, "NumRewards") or 0;
                if ( numItems > 0 and dbEntry:Contains(container, i, "Rewards") ) then
                    for index = 1, numItems do
                        text = text.."\t"..(dbEntry:GetValue(container, i, "Rewards", "Info"..index) or "");
                    end
                end
                _, name = dbEntry:GetValue(container, i, "RewardSpell");
                if ( name ) then
                    text = text.."\t"..name;
                end
                include = string.find(strlower(text), strlower(questLogFilter), 1, true);
            end
            if ( include ) then
                hasItems = true;
                table.insert(questLines, i);
            end
        end
    end
    
    if ( not hasItems ) then
        table.wipe(questLines);
    end
    
    dirty = false;
    owner = Armory:SelectedCharacter();

    return questLines;
end

local function GetQuestLineValue(index, key, subkey)
    local numLines = Armory:GetNumQuestLogEntries();
    if ( index > 0 and index <= numLines ) then
        local dbEntry = Armory.selectedDbBaseEntry;
        if ( subkey ) then
            return dbEntry:GetValue(container, questLines[index], key, subkey);
        elseif ( key ) then
            return dbEntry:GetValue(container, questLines[index], key);
        else
            return dbEntry:GetValue(container, questLines[index], "Info");
        end
    end
end

local function UpdateQuestHeaderState(index, isCollapsed)
    local dbEntry = Armory.selectedDbBaseEntry;

    if ( index == 0 ) then
        for i = 1, dbEntry:GetNumValues(container) do
            local name, _, _, _, isHeader = dbEntry:GetValue(container, i, "Info");
            if ( isHeader ) then
                Armory:SetHeaderLineState(container, name, isCollapsed);
            end
        end
    else
        local numLines = Armory:GetNumQuestLogEntries();
        if ( index > 0 and index <= numLines ) then
            local name = dbEntry:GetValue(container, questLines[index], "Info");
            Armory:SetHeaderLineState(container, name, isCollapsed);
        end
    end
    
    dirty = true;
end

----------------------------------------------------------
-- Quests Storage
----------------------------------------------------------

function Armory:UpdateQuests()
    local dbEntry = self.playerDbBaseEntry;
    local _, numQuests = _G.GetNumQuestLogEntries();

    if ( not self:IsLocked(container) ) then
        if ( self.ignoreQuestUpdate ) then
            self.ignoreQuestUpdate = false;
            return;
        end
        
        self:Lock(container);

        self:PrintDebug("UPDATE", container);

        if ( self:HasQuestLog() ) then
            self:SetQuests();

            dbEntry:SetValue(2, container, "NumQuests", numQuests);
            dbEntry:SetValue(2, container, "MaxDailyQuests", _G.GetMaxDailyQuests());
            dbEntry:SetValue(2, container, "DailyQuestsCompleted", _G.GetDailyQuestsCompleted());
        else
            dbEntry:SetValue(container, nil);
        end

        dirty = dirty or self:IsPlayerSelected();
        
        self:Unlock(container);
    else
        self:PrintDebug("LOCKED", container);
    end
end

function Armory:SetQuests()
    local dbEntry = self.playerDbBaseEntry;
    local currentQuest = _G.GetQuestLogSelection();
    local collapsed;

    -- store the complete (expanded) list
    local funcNumLines = _G.GetNumQuestLogEntries;
    local funcGetLineInfo = _G.GetQuestLogTitle;
    local funcGetLineState = function(index)
            local _, _, _, _, isHeader, isCollapsed = _G.GetQuestLogTitle(index);
            return isHeader, not isCollapsed;
        end;
    local funcExpand = _G.ExpandQuestHeader;
    local funcCollapse = _G.CollapseQuestHeader;
    local funcSelect = _G.SelectQuestLogEntry;
    local funcAdditionalInfo =
            function(index)
                return {
                    Link = _G.GetQuestLink,
                    Failed = _G.IsCurrentQuestFailed,
                    Text = _G.GetQuestLogQuestText,
                    TimeLeft = 
                        function(index) 
                            return _G.GetQuestLogTimeLeft(), time(); 
                        end,
                    RequiredMoney = _G.GetQuestLogRequiredMoney,
                    RewardMoney = _G.GetQuestLogRewardMoney,
                    RewardHonor = _G.GetQuestLogRewardHonor,
                    RewardSpell = _G.GetQuestLogRewardSpell,
                    RewardTalents = _G.GetQuestLogRewardTalents,
                    SpellLink = _G.GetQuestLogSpellLink,
                    RewardTitle = _G.GetQuestLogRewardTitle,
                    GroupNum = _G.GetQuestLogGroupNum,
                    NumLeaderBoards = _G.GetNumQuestLeaderBoards,
                    LeaderBoards =
                        function(index)
                            local data = {};
                            for i = 1, _G.GetNumQuestLeaderBoards() do
                                data[i] = ArmoryDbEntry.Save(_G.GetQuestLogLeaderBoard(i));
                            end
                            return data;
                        end,
                    NumRewards = _G.GetNumQuestLogRewards,
                    Rewards =
                        function(index)
                            local data = {};
                            for i = 1,  _G.GetNumQuestLogRewards() do
                                data["Info"..i] = ArmoryDbEntry.Save(_G.GetQuestLogRewardInfo(i));
                                data["Link"..i] = ArmoryDbEntry.Save(_G.GetQuestLogItemLink("reward", i));
                            end
                            return data;
                        end,
                    NumChoices = _G.GetNumQuestLogChoices,
                    Choices =
                        function(index)
                            local data = {};
                            for i = 1, _G.GetNumQuestLogChoices() do
                                data["Info"..i] = ArmoryDbEntry.Save(_G.GetQuestLogChoiceInfo(i));
                                data["Link"..i] = ArmoryDbEntry.Save(_G.GetQuestLogItemLink("choice", i));
                            end
                            return data;
                        end
               };
            end

    collapsed = dbEntry:SetExpandableListValues(container, funcNumLines, funcGetLineState, funcGetLineInfo, funcExpand, funcCollapse, funcAdditionalInfo, funcSelect);

    _G.SelectQuestLogEntry(currentQuest);

    -- if expand/collapse has been called a QUEST_LOG_UPDATE event will be fired immediately after the scan has been completed
    self.ignoreQuestUpdate = collapsed;
end

----------------------------------------------------------
-- Quests Interface
----------------------------------------------------------

function Armory:GetDailyQuestsCompleted()
    return self.selectedDbBaseEntry:GetValue(container, "DailyQuestsCompleted");
end

function Armory:GetMaxDailyQuests()
    return self.selectedDbBaseEntry:GetValue(container, "MaxDailyQuests");
end

function Armory:GetNumQuestLogEntries()
    if ( dirty or not self:IsSelectedCharacter(owner) ) then
        GetQuestLines();
    end
    local numQuests = self.selectedDbBaseEntry:GetValue(container, "NumQuests");
    return #questLines, numQuests;
end

function Armory:GetQuestLogTitle(index)
    local title, level, questTag, suggestedGroup, isHeader, isCollapsed = GetQuestLineValue(index);
    isCollapsed = self:GetHeaderLineState(container, title);
    return title, level, questTag, suggestedGroup, isHeader, isCollapsed;
end

function Armory:ExpandQuestHeader(index)
    UpdateQuestHeaderState(index, false);
end

function Armory:CollapseQuestHeader(index)
    UpdateQuestHeaderState(index, true);
end

function Armory:GetQuestLink(index)
    return GetQuestLineValue(index, "Link");
end

function Armory:GetQuestLogSelection()
    return selectedQuestLine;
end

function Armory:SelectQuestLogEntry(index)
    selectedQuestLine = index;
end

function Armory:IsCurrentQuestFailed()
    return GetQuestLineValue(selectedQuestLine, "Failed");
end

function Armory:GetQuestLogQuestText()
    return GetQuestLineValue(selectedQuestLine, "Text");
end

function Armory:GetQuestLogTimeLeft()
    local timeLeft, timestamp = GetQuestLineValue(selectedQuestLine, "TimeLeft");

    if ( timeLeft ) then
        timeLeft = timeLeft - (time() - timestamp);
        if ( timeLeft < 0 ) then
            timeLeft = 0;
        end
    end
    return timeLeft;
end

function Armory:GetQuestLogRequiredMoney()
    return GetQuestLineValue(selectedQuestLine, "RequiredMoney");
end

function Armory:GetQuestLogRewardMoney()
    return GetQuestLineValue(selectedQuestLine, "RewardMoney");
end

function Armory:GetQuestLogRewardHonor()
    return GetQuestLineValue(selectedQuestLine, "RewardHonor");
end

function Armory:GetQuestLogRewardSpell()
    return GetQuestLineValue(selectedQuestLine, "RewardSpell");
end

function Armory:GetQuestLogRewardTalents()
    return GetQuestLineValue(selectedQuestLine, "RewardTalents");
end

function Armory:GetQuestLogRewardTitle()
    return GetQuestLineValue(selectedQuestLine, "RewardTitle");
end

function Armory:GetQuestLogSpellLink()
    return GetQuestLineValue(selectedQuestLine, "SpellLink");
end

function Armory:GetQuestLogGroupNum()
    return GetQuestLineValue(selectedQuestLine, "GroupNum");
end

function Armory:GetNumQuestLeaderBoards()
    return GetQuestLineValue(selectedQuestLine, "NumLeaderBoards");
end

function Armory:GetQuestLogLeaderBoard(id)
    return GetQuestLineValue(selectedQuestLine, "LeaderBoards", id);
end

function Armory:GetNumQuestLogRewards()
    return GetQuestLineValue(selectedQuestLine, "NumRewards");
end

function Armory:GetQuestLogRewardInfo(id)
    return GetQuestLineValue(selectedQuestLine, "Rewards", "Info"..id);
end

function Armory:GetNumQuestLogChoices()
    return GetQuestLineValue(selectedQuestLine, "NumChoices");
end

function Armory:GetQuestLogChoiceInfo(id)
    return GetQuestLineValue(selectedQuestLine, "Choices", "Info"..id);
end

function Armory:GetQuestLogItemLink(itemType, id)
    if ( itemType == "reward" ) then
        return GetQuestLineValue(selectedQuestLine, "Rewards", "Link"..id);
    elseif ( itemType == "choice" ) then
        return GetQuestLineValue(selectedQuestLine, "Choices", "Link"..id);
    end
end

function Armory:SetQuestLogFilter(text)
    local refresh = (questLogFilter ~= text);
    questLogFilter = text;
    return refresh;
end

function Armory:GetQuestLogFilter()
    return questLogFilter;
end

----------------------------------------------------------
-- Find Methods
----------------------------------------------------------

function Armory:FindQuest(firstArg, ...)
    local dbEntry = self.selectedDbBaseEntry;
    local list = {};

    local numEntries = dbEntry:GetNumValues(container);
    if ( numEntries ) then
        local name, level, isHeader, link, text;
        for index = 1, numEntries do
            name, level, _, _, isHeader = dbEntry:GetValue(container, index, "Info");
            if ( not isHeader ) then
                link = dbEntry:GetValue(container, index, "Link");
                if ( self:GetConfigExtendedSearch() ) then
                    text = self:GetTextFromLink(link);
                else
                    text = name;
                end
                if ( self:FindTextParts(text, firstArg, ...) ) then
                    name = self:HexColor(ArmoryGetDifficultyColor(level))..name..FONT_COLOR_CODE_CLOSE;
                    table.insert(list, {label=QUEST_LOG, name=name, link=link});
                end
            end
        end
    end

    return list;
end

function Armory:FindQuestItem(itemList, firstArg, ...)
    local dbEntry = self.selectedDbBaseEntry;
    local list = itemList or {};
    
    local numEntries = dbEntry:GetNumValues(container);
    if ( numEntries ) then
        local questLogTitleText, level, isHeader;
        local text, label, name, link;

        for index = 1, numEntries do
            questLogTitleText, level, _, _, isHeader = dbEntry:GetValue(container, index, "Info");
            if ( not isHeader ) then
                label = ARMORY_CMD_FIND_QUEST_REWARD.." "..self:HexColor(ArmoryGetDifficultyColor(level))..questLogTitleText..FONT_COLOR_CODE_CLOSE;
                for i = 1, dbEntry:GetValue(container, index, "NumChoices") do
                    link = dbEntry:GetValue(container, index, "Choices", "Link"..i);
                    name = dbEntry:GetValue(container, index, "Choices", "Info"..i);
                    if ( self:GetConfigExtendedSearch() ) then
                        text = self:GetTextFromLink(link);
                    else
                        text = name;
                    end
                    if ( self:FindTextParts(text, firstArg, ...) ) then
                        table.insert(list, {label=label, name=name, link=link});
                    end
                end
                for i = 1, dbEntry:GetValue(container, index, "NumRewards") do
                    link = dbEntry:GetValue(container, index, "Rewards", "Link"..i);
                    name = dbEntry:GetValue(container, index, "Rewards", "Info"..i);
                    if ( self:GetConfigExtendedSearch() ) then
                        text = self:GetTextFromLink(link);
                    else
                        text = name;
                    end
                    if ( self:FindTextParts(text, firstArg, ...) ) then
                        table.insert(list, {label=label, name=name, link=link});
                    end
                end
            end
        end
    end
    
    return list;
end

function Armory:FindQuestSpell(spellList, firstArg, ...)
    local dbEntry = self.selectedDbBaseEntry;
    local list = spellList or {};

    local numEntries = dbEntry:GetNumValues(container);
    if ( numEntries ) then
        local questLogTitleText, level, isHeader;
        local text, label, name, link;

        for index = 1, numEntries do
            questLogTitleText, level, _, _, isHeader = dbEntry:GetValue(container, index, "Info");
            if ( not isHeader ) then
                if ( dbEntry:GetValue(container, index, "RewardSpell") ) then
                    _, name = dbEntry:GetValue(container, index, "RewardSpell");
                    link = dbEntry:GetValue(container, index, "SpellLink");
                    if ( self:GetConfigExtendedSearch() ) then
                        text = self:GetTextFromLink(link);
                    else
                        text = name;
                    end
                    if ( self:FindTextParts(text, firstArg, ...) ) then
                        label = ARMORY_CMD_FIND_QUEST_REWARD.." "..self:HexColor(ArmoryGetDifficultyColor(level))..questLogTitleText..FONT_COLOR_CODE_CLOSE;
                        table.insert(list, {label=label, name=name, link=link});
                    end
                end
            end
        end
    end
    
    return list;
end