--[[
    Armory Addon for World of Warcraft(tm).
    Revision: 136 2009-10-27T23:00:03Z
    URL: http://www.wow-neighbours.com

    License:
        This program is free software; you can redistribute it and/or
        modify it under the terms of the GNU General Public License
        as published by the Free Software Foundation; either version 2
        of the License, or (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program(see GPL.txt); if not, write to the Free Software
        Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    Note:
        This AddOn's source code is specifically designed to work with
        World of Warcraft's interpreted AddOn system.
        You have an implicit licence to use this AddOn with these facilities
        since that is it's designated purpose as per:
        http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
--]]

local Armory = Armory;
local QTip = LibStub("LibQTip-1.0");

local iconProvider, cellPrototype, baseCellPrototype = QTip:CreateCellProvider(QTip.LabelProvider)

function cellPrototype:InitializeCell()
    baseCellPrototype.InitializeCell(self);
end

function cellPrototype:SetupCell(tooltip, value)
    local _, height = baseCellPrototype.SetupCell(self, tooltip, format("|T%s:0|t", value), "CENTER");
    return baseCellPrototype.SetupCell(self, tooltip, format("|T%s:%d|t", value, height + 6), "CENTER");
end

local collapsed = {};
local totals = {};
local expiredMail;

local function GetCoinText(amount, spaced)
    local gold = floor(amount / 10000);
    local goldStr = GOLD_AMOUNT:gsub("%s*%%d%s*", "");
    local silver = mod(floor(amount / 100), 100);
    local silverStr = SILVER_AMOUNT:gsub("%s*%%d%s*", "");
    local copper = mod(floor(amount), 100);
    local copperStr = COPPER_AMOUNT:gsub("%s*%%d%s*", "");

    if ( (copperStr:byte(1) or 128) <= 127 ) then
        goldStr = goldStr:sub(1, 1):lower();
        silverStr = silverStr:sub(1, 1):lower();
        copperStr = copperStr:sub(1, 1):lower();
    end

    if ( amount >= 10000 ) then
        goldStr = format("|cffffffff%d|r|cffffd700%s|r", gold, goldStr);
    else
        goldStr = "";
    end
    if ( amount >= 100 ) then
        silverStr = format("|cffffffff%d|r|cffc7c7cf%s|r", silver, silverStr);
    else
        silverStr = "";
    end
    copperStr = format("|cffffffff%d|r|cffeda55f%s|r", copper, copperStr);
    
    if ( spaced ) then
        return strtrim(format("%s %s %s", goldStr, silverStr, copperStr));
    end
    return goldStr, silverStr, copperStr;
end

local function GetShortDate(value)
    if ( ARMORY_SHORTDATE_FORMAT:find("%", 1, true) ) then
        return date(ARMORY_SHORTDATE_FORMAT, value);
    end
    
    local dummy = time({year=3, month=2, day=1});
    local first = tonumber(date("%x", dummy):match("^(%d+)"));
    local day = date("%d", value):gsub("^0*", "");
    local month = date("%b", value);
    local year = "'"..date("%y", value);

    if ( first == 2 ) then
        return month.." "..day.." "..year;
    end
    return day.." "..month.." "..year;
end

local function OnRealmClick(self, realm)
    if ( collapsed[realm] ) then
        collapsed[realm] = nil;
    else
        collapsed[realm] = 1;
    end
    Armory:UpdateSummary();
end

local function OnCharacterClick(self, profile, button)
    if ( button == "RightButton" ) then
        if ( not Armory:IsPlayerSelected(profile) ) then
            local dialog = StaticPopup_Show("ARMORY_DELETE_CHARACTER", profile.character);
            dialog.data = profile;
            Armory:HideSummary();
        end
    else
        Armory:HideSummary();
        ArmoryFrameSelectCharacter(profile);
        if ( not ArmoryFrame:IsShown() ) then
            Armory:Toggle();
        end
    end
end

local function CellShowTooltip(self, text)
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 10, -10);
    GameTooltip:SetFrameLevel(self:GetFrameLevel() + 1);
    if ( type(text) == "table" ) then
        GameTooltip:SetText(text[1], 1.0, 1.0, 1.0);
        GameTooltip:AddLine(text[2]);
        for i = 3, #text do
            if ( text[i] ) then
                GameTooltip:AddLine(text[i], GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b);
            end
        end
    elseif ( text:find("|H") ) then
        GameTooltip:SetHyperlink(text);
    else
        GameTooltip:SetText(text);
    end
    GameTooltip:Show();
end

local function CellHideTooltip(self)
    if ( self.oldnewbie ~= nil ) then
        SHOW_NEWBIE_TIPS = self.oldnewbie;
        self.oldnewbie = nil;
    end
    GameTooltip:Hide();
end

local function OnMoneyEnter(self, info)
    GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
    GameTooltip:SetFrameLevel(self:GetFrameLevel() + 1);
    GameTooltip:AddLine(format(ARMORY_MONEY_TOTAL, info[1], info[2]), "", 1, 1, 1);
    SetTooltipMoney(GameTooltip, totals[info[1]][info[2]]);
    GameTooltip:Show();
end

local function OnExpiredEnter(self, expired)
    local tooltip = QTip:Acquire("ArmorySummaryExpiredTooltip", 3);
    local index, column, myColumn, name;

    local font = Armory.summary:GetHeaderFont();
    font:SetTextColor(GetTableColor(NORMAL_FONT_COLOR));
    
    tooltip:Clear();
    tooltip:SetFrameLevel(self:GetFrameLevel() + 1);
    tooltip:SetAutoHideDelay(0.1, self);
    tooltip:SmartAnchorTo(self);

    index, column = tooltip:AddHeader();
    myColumn = column; index, column = tooltip:SetCell(index, myColumn, INBOX, font, "LEFT", 2);
    myColumn = column; index, column = tooltip:SetCell(index, myColumn, ARMORY_EXPIRATION_LABEL, font, "RIGHT");
    tooltip:AddSeparator();

    for _, item in ipairs(expired) do
        name = Armory:GetColorFromLink(item.link)..item.name..FONT_COLOR_CODE_CLOSE.." ["..item.count.."]";
        if ( item.ignored ) then
            name = name..NORMAL_FONT_COLOR_CODE.." ("..IGNORED..")"..FONT_COLOR_CODE_CLOSE;
        end
        index, column = tooltip:AddLine();
        myColumn = column; index, column = tooltip:SetCell(index, myColumn, GetItemIcon(item.link), iconProvider);
        myColumn = column; index, column = tooltip:SetCell(index, myColumn, name);
        myColumn = column; index, column = tooltip:SetCell(index, myColumn, item.left, nil, "RIGHT");
    end

    tooltip:Show()
end

local function OnSkillClick(self, info)
    local currentProfile = Armory:CurrentProfile();
    local currentSkill = Armory:GetSelectedProfession();
    local link;

    Armory:SelectProfile(info);
    Armory:SetSelectedProfession(info.name);
    link = Armory:GetTradeSkillListLink();
    Armory:SetSelectedProfession(currentSkill);
    Armory:SelectProfile(currentProfile);
    
    if ( link ) then
        if ( IsShiftKeyDown() ) then
            if ( not ChatEdit_InsertLink(link) ) then
                ChatFrameEditBox:Show();
                ChatEdit_InsertLink(link);
            end
        else
            ShowUIPanel(ItemRefTooltip);
            if ( not ItemRefTooltip:IsShown() ) then
                ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE");
            end
            ItemRefTooltip:SetHyperlink(link:match("|H(.-)|h"));
            Armory:HideSummary();
        end
    end
end

function Armory:ShowSummary(parent)
    if ( self:GetConfigShowSummary() ) then
        local command = ArmoryCommand:new(Armory.InitializeSummary, self, parent);
        command:SetDelay(self:GetConfigSummaryDelay());
        command:Enforce();
        Armory.commandHandler:AddCommand(command);
    end
end

function Armory:InitializeSummary(parent)
    if ( self.summaryEnabled and not ((self.summary and self.summary:IsShown()) or ArmoryDropDownList1:IsVisible()) ) then
        local columns = 3;
        if ( self:GetConfigSummaryClass() ) then   
            columns = columns + 1;
        end
        if ( self:GetConfigSummaryLevel() ) then   
            columns = columns + 1;
        end
        if ( self:GetConfigSummaryZone() ) then
            columns = columns + 1;
        end
        if ( self:GetConfigSummaryXP() ) then   
            columns = columns + 1;
        end
        if ( self:GetConfigSummaryPlayed() ) then   
            columns = columns + 1;
        end
        if ( self:GetConfigSummaryOnline() ) then   
            columns = columns + 1;
        end
        if ( self:GetConfigSummaryMoney() ) then   
            columns = columns + 1;
        end
        if ( self:GetConfigSummaryAchievement() and self:HasAchievements() ) then   
            columns = columns + 1;
        end
        if ( self:GetConfigSummaryQuest() and self:HasQuestLog() ) then
            columns = columns + 1;
        end
        if ( self:GetConfigSummaryExpiration() and self:HasInventory() ) then
            columns = columns + 1;
        end
        if ( self:GetConfigSummaryTradeSkills() and self:HasTradeSkills() ) then  
            columns = columns + 5;
        end

        self.summary = QTip:Acquire("ArmorySummary", columns);
        self.summary:SetAutoHideDelay(0.5, parent);
        self.summary:SmartAnchorTo(parent);
        
        if ( self:HasInventory() ) then
            _, expiredMail = self:CheckMailItems(true);
        end
        
        self:UpdateSummary();
    end    
end

function Armory:UpdateSummary()
    GameTooltip:Hide();
    self.summary:Clear();

    local font = self.summary:GetHeaderFont();
    font:SetTextColor(GetTableColor(NORMAL_FONT_COLOR));
    
    local unknown = GRAY_FONT_COLOR_CODE..UNKNOWN..FONT_COLOR_CODE_CLOSE;

    local index, column = self.summary:AddHeader();
    local myColumn;

    myColumn = column; index, column = self.summary:SetCell(index, myColumn, "");    
    myColumn = column; index, column = self.summary:SetCell(index, myColumn, NAME, font, "LEFT", 2);
    if ( self:GetConfigSummaryClass() ) then   
        myColumn = column; index, column = self.summary:SetCell(index, myColumn, CLASS, font);
    end
    if ( self:GetConfigSummaryLevel() ) then   
        myColumn = column; index, column = self.summary:SetCell(index, myColumn, LEVEL_ABBR, font, "RIGHT");    
    end
    if ( self:GetConfigSummaryZone() ) then
        myColumn = column; index, column = self.summary:SetCell(index, myColumn, ZONE, font);
    end
    if ( self:GetConfigSummaryXP() ) then   
        myColumn = column; index, column = self.summary:SetCell(index, myColumn, XP, font, "CENTER");
    end
    if ( self:GetConfigSummaryPlayed() ) then   
        myColumn = column; index, column = self.summary:SetCell(index, myColumn, PLAYED, font, "CENTER");    
    end
    if ( self:GetConfigSummaryOnline() ) then   
        myColumn = column; index, column = self.summary:SetCell(index, myColumn, LASTONLINE, font, "RIGHT");
    end
    if ( self:GetConfigSummaryMoney() ) then   
        myColumn = column; index, column = self.summary:SetCell(index, myColumn, MONEY, font, "CENTER");
    end
    if ( self:GetConfigSummaryAchievement() and self:HasAchievements() ) then   
        myColumn = column; index, column = self.summary:SetCell(index, myColumn, "Interface\\Icons\\Achievement_Level_10", iconProvider);
        self.summary:SetCellScript(index, myColumn, "OnEnter", CellShowTooltip, ACHIEVEMENT_TITLE); 
        self.summary:SetCellScript(index, myColumn, "OnLeave", CellHideTooltip); 
    end
    if ( self:GetConfigSummaryQuest() and self:HasQuestLog() ) then
        myColumn = column; index, column = self.summary:SetCell(index, myColumn, "Interface\\Icons\\INV_Misc_Book_08", iconProvider);
        self.summary:SetCellScript(index, myColumn, "OnEnter", CellShowTooltip, QUESTS_LABEL); 
        self.summary:SetCellScript(index, myColumn, "OnLeave", CellHideTooltip); 
    end
    if ( self:GetConfigSummaryExpiration() and self:HasInventory() ) then
        myColumn = column; index, column = self.summary:SetCell(index, myColumn, "Interface\\Icons\\INV_Letter_15", iconProvider);
        self.summary:SetCellScript(index, myColumn, "OnEnter", CellShowTooltip, ARMORY_EXPIRATION_TITLE); 
        self.summary:SetCellScript(index, myColumn, "OnLeave", CellHideTooltip); 
    end
    if ( self:GetConfigSummaryTradeSkills() and self:HasTradeSkills() ) then  
        myColumn = column; index, column = self.summary:SetCell(index, myColumn, TRADE_SKILLS, font, "CENTER", 5);
    end

    self.summary:AddSeparator();

    local realms = self:RealmList();
    local currentProfile = self:CurrentProfile();
    local checked, factionIcon, class, classEn, timePlayed, lastPlayed, factionGroup, professions;
    local unit = "player";

    table.wipe(totals);
    
    for _, realm in ipairs(realms) do
        if ( realm ~= ARMORY_LOOKUP_REALM_ALIAS ) then
            totals[realm] = {};
            
            index, column = self.summary:AddLine();

            myColumn = column; 
            if ( #realms > 1 ) then
                index, column = self.summary:SetCell(index, myColumn, format("Interface\\Buttons\\UI-%sButton-Up", collapsed[realm] and "Plus" or "Minus"), iconProvider); 
                self.summary:SetCellScript(index, myColumn, "OnMouseDown", OnRealmClick, realm);
            else
                index, column = self.summary:SetCell(index, myColumn, "");
            end

            myColumn = column; index, column = self.summary:SetCell(index, myColumn, realm, nil, "LEFT", 2); 

            for _, character in ipairs(self:CharacterList(realm)) do
                local profile = {realm=realm, character=character};
                self:SelectProfile(profile);

                factionGroup = self:UnitFactionGroup(unit);
                if ( not totals[realm][factionGroup] ) then
                    totals[realm][factionGroup] = self:GetMoney();
                else
                    totals[realm][factionGroup] = totals[realm][factionGroup] + self:GetMoney();
                end

                if ( not collapsed[realm] ) then
                    checked = (realm == currentProfile.realm and character == currentProfile.character);

                    class, classEn = self:UnitClass(unit);
                    if ( classEn ) then
                        class = self:HexColor(GetTableColor(RAID_CLASS_COLORS[classEn]))..class..FONT_COLOR_CODE_CLOSE;
                    end
                    timePlayed, lastPlayed = self:GetTimePlayed();
                    skillIcons = "";
                    if ( factionGroup ) then
                        factionIcon = "Interface\\TargetingFrame\\UI-PVP-"..factionGroup;
                    else
                        factionIcon = "Interface\\Icons\\INV_Misc_QuestionMark";
                    end
                    
                    index, column = self.summary:AddLine();

                    myColumn = column; 
                    if ( checked ) then
                        index, column = self.summary:SetCell(index, myColumn, "Interface\\Buttons\\UI-CheckBox-Check", iconProvider);
                    else
                        index, column = self.summary:SetCell(index, myColumn, "");
                    end
                    
                    myColumn = column; index, column = self.summary:SetCell(index, myColumn, factionIcon, iconProvider);
                    myColumn = column; index, column = self.summary:SetCell(index, myColumn, NORMAL_FONT_COLOR_CODE..character..FONT_COLOR_CODE_CLOSE);
                    self.summary:SetCellScript(index, myColumn, "OnEnter", CellShowTooltip, {character, realm, ARMORY_SELECT_UNIT_HINT, ARMORY_DELETE_UNIT_HINT} ); 
                    self.summary:SetCellScript(index, myColumn, "OnLeave", CellHideTooltip); 
                    self.summary:SetCellScript(index, myColumn, "OnMouseDown", OnCharacterClick, profile);

                    if ( self:GetConfigSummaryClass() ) then   
                        myColumn = column; index, column = self.summary:SetCell(index, myColumn, class);
                    end
                    if ( self:GetConfigSummaryLevel() ) then   
                        myColumn = column; index, column = self.summary:SetCell(index, myColumn, self:UnitLevel(unit), nil, "CENTER");
                    end
                    if ( self:GetConfigSummaryZone() ) then   
                        myColumn = column; index, column = self.summary:SetCell(index, myColumn, self:GetZoneText());
                    end
                    if ( self:GetConfigSummaryXP() ) then   
                        myColumn = column; index, column = self.summary:SetCell(index, myColumn, self:GetXP() or NOT_APPLICABLE, nil, "CENTER");
                    end
                    if ( self:GetConfigSummaryPlayed() ) then   
                        myColumn = column; index, column = self.summary:SetCell(index, myColumn, (timePlayed and SecondsToTime(timePlayed, true)) or unknown, nil, "CENTER");
                    end
                    if ( self:GetConfigSummaryOnline() ) then   
                        myColumn = column; index, column = self.summary:SetCell(index, myColumn, (lastPlayed and GetShortDate(lastPlayed)) or unknown, nil, "CENTER");
                    end
                    if ( self:GetConfigSummaryMoney() ) then   
                        myColumn = column; index, column = self.summary:SetCell(index, myColumn, GetCoinText(self:GetMoney(), true), nil, "RIGHT");
                        self.summary:SetCellScript(index, myColumn, "OnEnter", OnMoneyEnter, {realm, factionGroup}); 
                        self.summary:SetCellScript(index, myColumn, "OnLeave", CellHideTooltip); 
                    end
                    if ( self:GetConfigSummaryAchievement() and self:HasAchievements() ) then   
                        myColumn = column; index, column = self.summary:SetCell(index, myColumn, self:GetTotalAchievementPoints() or "", nil, "CENTER");
                    end
                    if ( self:GetConfigSummaryQuest() and self:HasQuestLog() ) then
                        myColumn = column; index, column = self.summary:SetCell(index, myColumn, select(2, self:GetNumQuestLogEntries()) or "", nil, "CENTER");
                    end
                    if ( self:GetConfigSummaryExpiration() and self:HasInventory() ) then
                        local expired;
                        for profile, items in pairs(expiredMail) do
                            if ( realm == profile.realm and character == profile.character ) then
                                expired = items;
                                break;
                            end
                        end
                        myColumn = column; 
                        if ( expired ) then
                            local count = 0;
                            for _, item in ipairs(expired) do
                                if ( not item.ignored ) then
                                    count = count + 1;
                                end
                            end
                            index, column = self.summary:SetCell(index, myColumn, count.."/"..#expired, nil, "CENTER");
                            self.summary:SetCellScript(index, myColumn, "OnEnter", OnExpiredEnter, expired);
                        else
                            index, column = self.summary:SetCell(index, myColumn, "");
                        end
                    end
                    if ( self:GetConfigSummaryTradeSkills() and self:HasTradeSkills() ) then
                        for _, name in ipairs(self:GetProfessionNames()) do
                            myColumn = column; index, column = self.summary:SetCell(index, myColumn, self:GetProfessionTexture(name), iconProvider);
                            self.summary:SetCellScript(index, myColumn, "OnEnter", CellShowTooltip, {character, name, ARMORY_SUMMARY_TRADESKILL_HINT}); 
                            self.summary:SetCellScript(index, myColumn, "OnLeave", CellHideTooltip); 
                            self.summary:SetCellScript(index, myColumn, "OnMouseDown", OnSkillClick, {name=name, realm=realm, character=character});
                        end
                    end
                end
            end
        end
    end
    self:SelectProfile(currentProfile);
    
    self.summary:UpdateScrolling(512);
    self.summary:Show();
end

function Armory:HideSummary()
    if ( self.summary ) then
        QTip:Release(self.summary);
        Armory.summary = nil;
    end
end
